@@ -14,9 +14,9 @@
 
 #include "application.h"
 #include "owncloudgui.h"
-#include "ocsnavigationappsjob.h"
 #include "theme.h"
 #include "folderman.h"
+#include "configfile.h"
 #include "progressdispatcher.h"
 #include "owncloudsetupwizard.h"
 #include "sharedialog.h"
@@ -27,51 +27,37 @@
 #include "accountstate.h"
 #include "openfilemanager.h"
 #include "accountmanager.h"
+#include "aboutdialog.h"
 #include "common/syncjournalfilerecord.h"
 #include "creds/abstractcredentials.h"
-#ifdef WITH_LIBCLOUDPROVIDERS
-#include "cloudproviders/cloudprovidermanager.h"
-#endif
+#include "guiutility.h"
 
 #include <QDesktopServices>
 #include <QDir>
 #include <QMessageBox>
-#include <QSignalMapper>
+#include <QDialog>
+#include <QHBoxLayout>
+#include <QScreen>
+
 #ifdef WITH_LIBCLOUDPROVIDERS
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusInterface>
+#include "libcloudproviders/libcloudproviders.h"
 #endif
 
-#if defined(Q_OS_X11)
-#include <QX11Info>
-#endif
-
-#include <QJsonDocument>
-#include <QJsonObject>
-#include <QJsonArray>
-
 namespace OCC {
 
 const char propertyAccountC[] = "oc_account";
 
 ownCloudGui::ownCloudGui(Application *parent)
     : QObject(parent)
-    , _tray(nullptr)
+    , _tray(new Systray(this))
     , _settingsDialog(new SettingsDialog(this))
-    , _logBrowser(nullptr)
-#ifdef WITH_LIBCLOUDPROVIDERS
-    , _bus(QDBusConnection::sessionBus())
-#endif
     , _recentActionsMenu(nullptr)
     , _app(parent)
 {
-    _tray = new Systray();
-    _tray->setParent(this);
-
     // for the beginning, set the offline icon until the account was verified
     _tray->setIcon(Theme::instance()->folderOfflineIcon(/*systray?*/ true, /*currently visible?*/ false));
 
-    connect(_tray.data(), &QSystemTrayIcon::activated,
+    connect(_tray, &QSystemTrayIcon::activated,
         this, &ownCloudGui::slotTrayClicked);
 
     setupActions();
@@ -79,6 +65,12 @@ ownCloudGui::ownCloudGui(Application *parent)
 
     _tray->show();
 
+#ifdef WITH_LIBCLOUDPROVIDERS
+    auto exporter = new LibCloudProviders(this);
+    exporter->start();
+    connect(exporter, &LibCloudProviders::showSettings, this, &ownCloudGui::slotShowSettings);
+#endif
+
     ProgressDispatcher *pd = ProgressDispatcher::instance();
     connect(pd, &ProgressDispatcher::progressInfo, this,
         &ownCloudGui::slotUpdateProgress);
@@ -91,50 +83,26 @@ ownCloudGui::ownCloudGui(Application *parent)
         this, &ownCloudGui::updateContextMenuNeeded);
     connect(AccountManager::instance(), &AccountManager::accountRemoved,
         this, &ownCloudGui::updateContextMenuNeeded);
-
-    connect(Logger::instance(), &Logger::guiLog,
-        this, &ownCloudGui::slotShowTrayMessage);
-    connect(Logger::instance(), &Logger::optionalGuiLog,
-        this, &ownCloudGui::slotShowOptionalTrayMessage);
-    connect(Logger::instance(), &Logger::guiMessage,
-        this, &ownCloudGui::slotShowGuiMessage);
 }
 
-#ifdef WITH_LIBCLOUDPROVIDERS
-void ownCloudGui::setupCloudProviders()
+ownCloudGui::~ownCloudGui()
 {
-    new CloudProviderManager(this);
+    _settingsDialog->deleteLater();
 }
 
-bool ownCloudGui::cloudProviderApiAvailable()
-{
-    if (!_bus.isConnected()) {
-        return false;
-    }
-    QDBusInterface dbus_iface("org.freedesktop.CloudProviderManager", "/org/freedesktop/CloudProviderManager",
-                              "org.freedesktop.CloudProvider.Manager1", _bus);
-
-    if (!dbus_iface.isValid()) {
-        qCInfo(lcApplication) << "DBus interface unavailable";
-        return false;
-    }
-    return true;
-}
-#endif
-
 // This should rather be in application.... or rather in ConfigFile?
 void ownCloudGui::slotOpenSettingsDialog()
 {
     // if account is set up, start the configuration wizard.
     if (!AccountManager::instance()->accounts().isEmpty()) {
-        if (_settingsDialog.isNull() || QApplication::activeWindow() != _settingsDialog) {
+        if (QApplication::activeWindow() != _settingsDialog) {
             slotShowSettings();
         } else {
             _settingsDialog->close();
         }
     } else {
         qCInfo(lcApplication) << "No configured folders yet, starting setup wizard";
-        slotNewAccountWizard();
+        runNewAccountWizard();
     }
 }
 
@@ -150,26 +118,16 @@ void ownCloudGui::slotTrayClicked(QSystemTrayIcon::ActivationReason reason)
 
     // Left click
     if (reason == QSystemTrayIcon::Trigger) {
-        if (OwncloudSetupWizard::bringWizardToFrontIfVisible()) {
-            // brought wizard to front
-        } else if (_shareDialogs.size() > 0) {
-            // Share dialog(s) be hidden by other apps, bring them back
-            Q_FOREACH (const QPointer<ShareDialog> &shareDialog, _shareDialogs) {
-                Q_ASSERT(shareDialog.data());
-                raiseDialog(shareDialog);
-            }
-        } else {
 #ifdef Q_OS_MAC
-            // on macOS, a left click always opens menu.
-            // However if the settings dialog is already visible but hidden
-            // by other applications, this will bring it to the front.
-            if (!_settingsDialog.isNull() && _settingsDialog->isVisible()) {
-                raiseDialog(_settingsDialog.data());
-            }
-#else
-            slotOpenSettingsDialog();
-#endif
+        // on macOS, a left click always opens menu.
+        // However if the settings dialog is already visible but hidden
+        // by other applications, this will bring it to the front.
+        if (_settingsDialog->isVisible()) {
+            raiseDialog(_settingsDialog);
         }
+#else
+        slotOpenSettingsDialog();
+#endif
     }
     // FIXME: Also make sure that any auto updater dialogue https://github.com/owncloud/client/issues/5613
     // or SSL error dialog also comes to front.
@@ -222,7 +180,7 @@ void ownCloudGui::slotTrayMessageIfServerUnsupported(Account *account)
     if (account->serverVersionUnsupported()) {
         slotShowTrayMessage(
             tr("Unsupported Server Version"),
-            tr("The server on account %1 runs an old and unsupported version %2. "
+            tr("The server on account %1 runs an unsupported version %2. "
                "Using this client with unsupported server versions is untested and "
                "potentially dangerous. Proceed at your own risk.")
                 .arg(account->displayName(), account->serverVersion()));
@@ -237,12 +195,12 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     QVector<AccountStatePtr> problemAccounts;
     auto setStatusText = [&](const QString &text) {
         // Don't overwrite the status if we're currently syncing
-        if (FolderMan::instance()->currentSyncFolder())
+        if (FolderMan::instance()->isAnySyncRunning())
             return;
         _actionStatus->setText(text);
     };
 
-    foreach (auto a, AccountManager::instance()->accounts()) {
+    for (const auto &a : AccountManager::instance()->accounts()) {
         if (!a->isSignedOut()) {
             allSignedOut = false;
         }
@@ -252,7 +210,9 @@ void ownCloudGui::slotComputeOverallSyncStatus()
             allDisconnected = false;
         }
     }
-    foreach (Folder *f, FolderMan::instance()->map()) {
+
+    const auto &map = FolderMan::instance()->map();
+    for (auto *f : map) {
         if (!f->syncPaused()) {
             allPaused = false;
         }
@@ -268,14 +228,14 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 #ifdef Q_OS_WIN
         // Windows has a 128-char tray tooltip length limit.
         QStringList accountNames;
-        foreach (AccountStatePtr a, problemAccounts) {
+        for (const auto &a : qAsConst(problemAccounts)) {
             accountNames.append(a->account()->displayName());
         }
         _tray->setToolTip(tr("Disconnected from %1").arg(accountNames.join(QLatin1String(", "))));
 #else
         QStringList messages;
         messages.append(tr("Disconnected from accounts:"));
-        foreach (AccountStatePtr a, problemAccounts) {
+        for (const auto &a : qAsConst(problemAccounts)) {
             QString message = tr("Account %1: %2").arg(a->account()->displayName(), a->stateString(a->state()));
             if (!a->connectionErrors().empty()) {
                 message += QLatin1String("\n");
@@ -302,60 +262,51 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 
     // display the info of the least successful sync (eg. do not just display the result of the latest sync)
     QString trayMessage;
-    FolderMan *folderMan = FolderMan::instance();
-    Folder::Map map = folderMan->map();
 
-    SyncResult::Status overallStatus = SyncResult::Undefined;
-    bool hasUnresolvedConflicts = false;
-    FolderMan::trayOverallStatus(map.values(), &overallStatus, &hasUnresolvedConflicts);
+    auto trayOverallStatusResult = FolderMan::trayOverallStatus(map.values());
 
-    // If the sync succeeded but there are unresolved conflicts,
-    // show the problem icon!
-    auto iconStatus = overallStatus;
-    if (iconStatus == SyncResult::Success && hasUnresolvedConflicts) {
-        iconStatus = SyncResult::Problem;
-    }
-
-    // If we don't get a status for whatever reason, that's a Problem
-    if (iconStatus == SyncResult::Undefined) {
-        iconStatus = SyncResult::Problem;
-    }
-
-    QIcon statusIcon = Theme::instance()->syncStateIcon(iconStatus, true, contextMenuVisible());
+    const QIcon statusIcon = Theme::instance()->syncStateIcon(trayOverallStatusResult.overallStatus(), true, contextMenuVisible());
     _tray->setIcon(statusIcon);
 
     // create the tray blob message, check if we have an defined state
-    if (map.count() > 0) {
 #ifdef Q_OS_WIN
-        // Windows has a 128-char tray tooltip length limit.
-        trayMessage = folderMan->trayTooltipStatusString(overallStatus, hasUnresolvedConflicts, false);
+    // Windows has a 128-char tray tooltip length limit.
+    trayMessage = FolderMan::instance()->trayTooltipStatusString(trayOverallStatusResult.overallStatus(), false);
 #else
-        QStringList allStatusStrings;
-        foreach (Folder *folder, map.values()) {
-            QString folderMessage = FolderMan::trayTooltipStatusString(
-                folder->syncResult().status(),
-                folder->syncResult().hasUnresolvedConflicts(),
-                folder->syncPaused());
-            allStatusStrings += tr("Folder %1: %2").arg(folder->shortGuiLocalPath(), folderMessage);
-        }
-        trayMessage = allStatusStrings.join(QLatin1String("\n"));
+    QStringList allStatusStrings;
+    for (auto *folder : map) {
+        QString folderMessage = FolderMan::trayTooltipStatusString(
+            folder->syncResult(),
+            folder->syncPaused());
+        allStatusStrings += tr("Folder %1: %2").arg(folder->shortGuiLocalPath(), folderMessage);
+    }
+    trayMessage = allStatusStrings.join(QLatin1String("\n"));
 #endif
-        _tray->setToolTip(trayMessage);
+    _tray->setToolTip(trayMessage);
 
-        if (overallStatus == SyncResult::Success || overallStatus == SyncResult::Problem) {
-            if (hasUnresolvedConflicts) {
-                setStatusText(tr("Unresolved conflicts"));
-            } else {
-                setStatusText(tr("Up to date"));
-            }
-        } else if (overallStatus == SyncResult::Paused) {
-            setStatusText(tr("Synchronization is paused"));
-        } else {
-            setStatusText(tr("Error during synchronization"));
+    switch (trayOverallStatusResult.overallStatus().status()) {
+    case SyncResult::Problem:
+        if (trayOverallStatusResult.overallStatus().hasUnresolvedConflicts()) {
+            setStatusText(tr("Unresolved %1 conflicts").arg(QString::number(trayOverallStatusResult.overallStatus().numNewConflictItems())));
+        } else if (trayOverallStatusResult.overallStatus().numBlacklistErrors() != 0) {
+            setStatusText(tr("Ignored errors %1").arg(QString::number(trayOverallStatusResult.overallStatus().numBlacklistErrors())));
         }
-    } else {
+        break;
+    case SyncResult::Success: {
+        QString lastSyncDoneString;
+        // display only the time in case the last sync was today
+        if (QDateTime::currentDateTime().date() == trayOverallStatusResult.lastSyncDone.date()) {
+            lastSyncDoneString = QLocale().toString(trayOverallStatusResult.lastSyncDone.time());
+        } else {
+            lastSyncDoneString = QLocale().toString(trayOverallStatusResult.lastSyncDone);
+        }
+        setStatusText(tr("Up to date (%1)").arg(lastSyncDoneString));
+    } break;
+    case SyncResult::Undefined:
         _tray->setToolTip(tr("There are no sync folders configured."));
         setStatusText(tr("No sync folders configured"));
+    default:
+        setStatusText(FolderMan::instance()->trayTooltipStatusString(trayOverallStatusResult.overallStatus(), false));
     }
 }
 
@@ -373,10 +324,11 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
 
     FolderMan *folderMan = FolderMan::instance();
     bool firstFolder = true;
-    bool singleSyncFolder = folderMan->map().size() == 1 && Theme::instance()->singleSyncFolder();
+    const auto &map = folderMan->map();
+    bool singleSyncFolder = map.size() == 1 && Theme::instance()->singleSyncFolder();
     bool onePaused = false;
     bool allPaused = true;
-    foreach (Folder *folder, folderMan->map()) {
+    for (auto *folder : map) {
         if (folder->accountState() != accountState.data()) {
             continue;
         }
@@ -401,7 +353,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
     menu->addSeparator();
     if (separateMenu) {
         if (onePaused) {
-            QAction *enable = menu->addAction(tr("Resume all folders"));
+            QAction *enable = menu->addAction(tr("Unpause all folders"));
             enable->setProperty(propertyAccountC, QVariant::fromValue(accountState));
             connect(enable, &QAction::triggered, this, &ownCloudGui::slotUnpauseAllFolders);
         }
@@ -412,7 +364,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
         }
 
         if (accountState->isSignedOut()) {
-            QAction *signin = menu->addAction(tr("Log in …"));
+            QAction *signin = menu->addAction(tr("Log in..."));
             signin->setProperty(propertyAccountC, QVariant::fromValue(accountState));
             connect(signin, &QAction::triggered, this, &ownCloudGui::slotLogin);
         } else {
@@ -423,6 +375,11 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
     }
 }
 
+SettingsDialog *ownCloudGui::settingsDialog() const
+{
+    return _settingsDialog;
+}
+
 void ownCloudGui::slotContextMenuAboutToShow()
 {
     _contextMenuVisibleManual = true;
@@ -452,6 +409,12 @@ bool ownCloudGui::contextMenuVisible() const
     return _contextMenu->isVisible();
 }
 
+void ownCloudGui::hideAndShowTray()
+{
+    _tray->hide();
+    _tray->show();
+}
+
 static bool minimalTrayMenu()
 {
     static QByteArray var = qgetenv("OWNCLOUD_MINIMAL_TRAY_MENU");
@@ -520,6 +483,10 @@ void ownCloudGui::setupContextMenu()
     // The tray menu is surprisingly problematic. Being able to switch to
     // a minimal version of it is a useful workaround and testing tool.
     if (minimalTrayMenu()) {
+        if (! Theme::instance()->about().isEmpty()) {
+            _contextMenu->addSeparator();
+            _contextMenu->addAction(_actionAbout);
+        }
         _contextMenu->addAction(_actionQuit);
         return;
     }
@@ -580,9 +547,9 @@ void ownCloudGui::setupContextMenu()
     _delayedTrayUpdateTimer.setInterval(2 * 1000);
     _delayedTrayUpdateTimer.setSingleShot(true);
 
-    connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(slotContextMenuAboutToShow()));
+    connect(_contextMenu.data(), &QMenu::aboutToShow, this, &ownCloudGui::slotContextMenuAboutToShow);
     // unfortunately aboutToHide is unreliable, it seems to work on OSX though
-    connect(_contextMenu.data(), SIGNAL(aboutToHide()), SLOT(slotContextMenuAboutToHide()));
+    connect(_contextMenu.data(), &QMenu::aboutToHide, this, &ownCloudGui::slotContextMenuAboutToHide);
 
     // Populate the context menu now.
     updateContextMenu();
@@ -619,23 +586,32 @@ void ownCloudGui::updateContextMenu()
     slotRebuildRecentMenus();
 
     // We must call deleteLater because we might be called from the press in one of the actions.
-    foreach (auto menu, _accountMenus) {
+    for (auto *menu : qAsConst(_accountMenus)) {
         menu->deleteLater();
     }
     _accountMenus.clear();
 
-    auto accountList = AccountManager::instance()->accounts();
+
+    const auto &accountList = AccountManager::instance()->accounts();
 
     bool isConfigured = (!accountList.isEmpty());
     bool atLeastOneConnected = false;
+    bool atLeastOneSignedOut = false;
+    bool atLeastOneSignedIn = false;
     bool atLeastOnePaused = false;
     bool atLeastOneNotPaused = false;
-    foreach (auto a, accountList) {
+    for (const auto &a : accountList) {
         if (a->isConnected()) {
             atLeastOneConnected = true;
         }
+        if (a->isSignedOut()) {
+            atLeastOneSignedOut = true;
+        } else {
+            atLeastOneSignedIn = true;
+        }
     }
-    foreach (auto f, FolderMan::instance()->map()) {
+
+    for (auto *f : FolderMan::instance()->map()) {
         if (f->syncPaused()) {
             atLeastOnePaused = true;
         } else {
@@ -644,48 +620,40 @@ void ownCloudGui::updateContextMenu()
     }
 
     if (accountList.count() > 1) {
-        foreach (AccountStatePtr account, accountList) {
+        for (const auto &account : accountList) {
             QMenu *accountMenu = new QMenu(account->account()->displayName(), _contextMenu.data());
             _accountMenus.append(accountMenu);
             _contextMenu->addMenu(accountMenu);
 
             addAccountContextMenu(account, accountMenu, true);
-            fetchNavigationApps(account);
         }
     } else if (accountList.count() == 1) {
         addAccountContextMenu(accountList.first(), _contextMenu.data(), false);
-        fetchNavigationApps(accountList.first());
     }
 
-    _contextMenu->addSeparator();
-
     _contextMenu->addAction(_actionStatus);
     if (isConfigured && atLeastOneConnected) {
         _contextMenu->addMenu(_recentActionsMenu);
     }
 
-    _contextMenu->addSeparator();
-
     if (accountList.isEmpty()) {
         _contextMenu->addAction(_actionNewAccountWizard);
     }
     _contextMenu->addAction(_actionSettings);
-    if (!Theme::instance()->helpUrl().isEmpty()) {
-        _contextMenu->addAction(_actionHelp);
-    }
 
     if (_actionCrash) {
         _contextMenu->addAction(_actionCrash);
+        _contextMenu->addAction(_actionCrashEnforce);
+        _contextMenu->addAction(_actionCrashFatal);
     }
 
     _contextMenu->addSeparator();
-
     if (atLeastOnePaused) {
         QString text;
         if (accountList.count() > 1) {
-            text = tr("Resume all synchronization");
+            text = tr("Unpause all synchronization");
         } else {
-            text = tr("Resume synchronization");
+            text = tr("Unpause synchronization");
         }
         QAction *action = _contextMenu->addAction(text);
         connect(action, &QAction::triggered, this, &ownCloudGui::slotUnpauseAllFolders);
@@ -700,6 +668,12 @@ void ownCloudGui::updateContextMenu()
         QAction *action = _contextMenu->addAction(text);
         connect(action, &QAction::triggered, this, &ownCloudGui::slotPauseAllFolders);
     }
+
+    if (! Theme::instance()->about().isEmpty()) {
+        _contextMenu->addSeparator();
+        _contextMenu->addAction(_actionAbout);
+    }
+
     _contextMenu->addAction(_actionQuit);
 
     if (_workaroundShowAndHideTray) {
@@ -728,17 +702,18 @@ void ownCloudGui::updateContextMenuNeeded()
     }
 }
 
-void ownCloudGui::slotShowTrayMessage(const QString &title, const QString &msg)
+void ownCloudGui::slotShowTrayMessage(const QString &title, const QString &msg, const QIcon &icon)
 {
-    if (_tray)
-        _tray->showMessage(title, msg);
-    else
-        qCWarning(lcApplication) << "Tray not ready: " << msg;
+    // SyncResult::Problem is returns the info icon
+    _tray->showMessage(title, msg, icon.isNull() ? Theme::instance()->syncStateIcon(SyncResult::Problem) : icon);
 }
 
-void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QString &msg)
+void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QString &msg, const QIcon &icon)
 {
-    slotShowTrayMessage(title, msg);
+    ConfigFile cfg;
+    if (cfg.optionalDesktopNotifications()) {
+        slotShowTrayMessage(title, msg, icon);
+    }
 }
 
 
@@ -769,121 +744,45 @@ void ownCloudGui::setupActions()
 {
     _actionStatus = new QAction(tr("Unknown status"), this);
     _actionStatus->setEnabled(false);
-    _navLinksMenu = new QMenu(tr("Apps"));
-    _navLinksMenu->setEnabled(false);
-    _actionSettings = new QAction(tr("Settings …"), this);
-    _actionNewAccountWizard = new QAction(tr("New account …"), this);
-    _actionRecent = new QAction(tr("View more activity …"), this);
+    _actionSettings = new QAction(tr("Settings..."), this);
+    _actionNewAccountWizard = new QAction(tr("New account..."), this);
+    _actionRecent = new QAction(tr("Details..."), this);
     _actionRecent->setEnabled(true);
 
     QObject::connect(_actionRecent, &QAction::triggered, this, &ownCloudGui::slotShowSyncProtocol);
     QObject::connect(_actionSettings, &QAction::triggered, this, &ownCloudGui::slotShowSettings);
-    QObject::connect(_actionNewAccountWizard, &QAction::triggered, this, &ownCloudGui::slotNewAccountWizard);
+    QObject::connect(_actionNewAccountWizard, &QAction::triggered, this, &ownCloudGui::runNewAccountWizard);
     _actionHelp = new QAction(tr("Help"), this);
     QObject::connect(_actionHelp, &QAction::triggered, this, &ownCloudGui::slotHelp);
+    _actionAbout = new QAction(tr("About %1").arg(Theme::instance()->appNameGUI()), this);
+    QObject::connect(_actionAbout, &QAction::triggered, this, &ownCloudGui::slotAbout);
     _actionQuit = new QAction(tr("Quit %1").arg(Theme::instance()->appNameGUI()), this);
-    QObject::connect(_actionQuit, SIGNAL(triggered(bool)), _app, SLOT(quit()));
+    QObject::connect(_actionQuit, &QAction::triggered, _app, &QApplication::quit);
+
+    _actionLogin = new QAction(tr("Log in..."), this);
+    connect(_actionLogin, &QAction::triggered, this, &ownCloudGui::slotLogin);
+    _actionLogout = new QAction(tr("Log out"), this);
+    connect(_actionLogout, &QAction::triggered, this, &ownCloudGui::slotLogout);
 
     if (_app->debugMode()) {
-        _actionCrash = new QAction(tr("Crash now", "Only shows in debug mode to allow testing the crash handler"), this);
+        _actionCrash = new QAction("Crash now - Div by zero", this);
         connect(_actionCrash, &QAction::triggered, _app, &Application::slotCrash);
+        _actionCrashEnforce = new QAction("Crash now - ENFORCE()", this);
+        connect(_actionCrashEnforce, &QAction::triggered, _app, &Application::slotCrashEnforce);
+        _actionCrashFatal = new QAction("Crash now - qFatal", this);
+        connect(_actionCrashFatal, &QAction::triggered, _app, &Application::slotCrashFatal);
     } else {
         _actionCrash = nullptr;
+        _actionCrashEnforce = nullptr;
+        _actionCrashFatal = nullptr;
     }
 }
 
-void ownCloudGui::slotEtagResponseHeaderReceived(const QByteArray &value, int statusCode){
-    if(statusCode == 200){
-        qCDebug(lcApplication) << "New navigation apps ETag Response Header received " << value;
-        auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC));
-        account->setNavigationAppsEtagResponseHeader(value);
-    }
-}
-
-void ownCloudGui::fetchNavigationApps(AccountStatePtr account){
-    OcsNavigationAppsJob *job = new OcsNavigationAppsJob(account->account());
-    job->setProperty(propertyAccountC, QVariant::fromValue(account));
-    job->addRawHeader("If-None-Match", account->navigationAppsEtagResponseHeader());
-    connect(job, &OcsNavigationAppsJob::appsJobFinished, this, &ownCloudGui::slotNavigationAppsFetched);
-    connect(job, &OcsNavigationAppsJob::etagResponseHeaderReceived, this, &ownCloudGui::slotEtagResponseHeaderReceived);
-    connect(job, &OcsNavigationAppsJob::ocsError, this, &ownCloudGui::slotOcsError);
-    job->getNavigationApps();
-}
-
-void ownCloudGui::buildNavigationAppsMenu(AccountStatePtr account, QMenu *accountMenu){
-    auto navLinks = _navApps.value(account);
-
-    _navLinksMenu->clear();
-    _navLinksMenu->setEnabled(navLinks.size() > 0);
-
-    if(navLinks.size() > 0){
-        // when there is only one account add the nav links above the settings
-        QAction *actionBefore = _actionSettings;
-
-        // when there is more than one account add the nav links above pause/unpause folder or logout action
-        if(AccountManager::instance()->accounts().size() > 1){
-            foreach(QAction *action, accountMenu->actions()){
-
-                // pause/unpause folder and logout actions have propertyAccountC
-                if(auto actionAccount = qvariant_cast<AccountStatePtr>(action->property(propertyAccountC))){
-                    if(actionAccount == account){
-                        actionBefore = action;
-                        break;
-                    }
-                }
-            }
-        }
-
-        // Create submenu with links
-        foreach (const QJsonValue &value, navLinks) {
-            auto navLink = value.toObject();
-            QAction *action = new QAction(navLink.value("name").toString(), this);
-            QUrl href(navLink.value("href").toString());
-            connect(action, &QAction::triggered, this, [href] { QDesktopServices::openUrl(href); });
-            _navLinksMenu->addAction(action);
-        }
-    }
-}
-
-void ownCloudGui::slotNavigationAppsFetched(const QJsonDocument &reply, int statusCode)
-{
-    if(auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))){
-        if (statusCode == 304) {
-            qCWarning(lcApplication) << "Status code " << statusCode << " Not Modified - No new navigation apps.";
-        } else {
-            if(!reply.isEmpty()){
-                auto element = reply.object().value("ocs").toObject().value("data");
-                auto navLinks = element.toArray();
-                _navApps.insert(account, navLinks);
-            }
-        }
-
-        // TODO see pull #523
-        auto accountList = AccountManager::instance()->accounts();
-        if(accountList.size() > 1){
-            // the list of apps will be displayed under the account that it belongs to
-            foreach (QMenu *accountMenu, _accountMenus) {
-                if(accountMenu->title() == account->account()->displayName()){
-                    buildNavigationAppsMenu(account, accountMenu);
-                    break;
-                }
-            }
-        } else if(accountList.size() == 1){
-            buildNavigationAppsMenu(account, _contextMenu.data());
-        }
-    }
-}
-
-void ownCloudGui::slotOcsError(int statusCode, const QString &message)
-{
-    emit serverError(statusCode, message);
-}
-
 void ownCloudGui::slotRebuildRecentMenus()
 {
     _recentActionsMenu->clear();
     if (!_recentItemsActions.isEmpty()) {
-        foreach (QAction *a, _recentItemsActions) {
+        for (auto *a : qAsConst(_recentItemsActions)) {
             _recentActionsMenu->addAction(a);
         }
         _recentActionsMenu->addSeparator();
@@ -924,11 +823,11 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
     }
 
     if (progress.totalSize() == 0) {
-        quint64 currentFile = progress.currentFile();
-        quint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
+        qint64 currentFile = progress.currentFile();
+        qint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
         QString msg;
         if (progress.trustEta()) {
-            msg = tr("Syncing %1 of %2 (%3 left)")
+            msg = tr("Syncing %1 of %2  (%3 left)")
                       .arg(currentFile)
                       .arg(totalFileCount)
                       .arg(Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta));
@@ -957,8 +856,7 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
         && shouldShowInRecentsMenu(progress._lastCompletedItem)) {
         if (Progress::isWarningKind(progress._lastCompletedItem._status)) {
             // display a warn icon if warnings happened.
-            QIcon warnIcon(":/client/resources/warning");
-            _actionRecent->setIcon(warnIcon);
+            _actionRecent->setIcon(Utility::getCoreIcon(QStringLiteral("warning")));
         }
 
         QString kindStr = Progress::asResultString(progress._lastCompletedItem);
@@ -993,8 +891,7 @@ void ownCloudGui::slotLogin()
         account->account()->resetRejectedCertificates();
         account->signIn();
     } else {
-        auto list = AccountManager::instance()->accounts();
-        foreach (const auto &a, list) {
+        for (const auto &a : AccountManager::instance()->accounts()) {
             a->signIn();
         }
     }
@@ -1005,10 +902,10 @@ void ownCloudGui::slotLogout()
     auto list = AccountManager::instance()->accounts();
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
         list.clear();
-        list.append(account);
+        list.insert(account->account()->uuid(), account);
     }
 
-    foreach (const auto &ai, list) {
+    for (const auto &ai : qAsConst(list)) {
         ai->signOutByUi();
     }
 }
@@ -1023,9 +920,16 @@ void ownCloudGui::slotPauseAllFolders()
     setPauseOnAllFoldersHelper(true);
 }
 
-void ownCloudGui::slotNewAccountWizard()
+void ownCloudGui::runNewAccountWizard()
 {
-    OwncloudSetupWizard::runWizard(qApp, SLOT(slotownCloudWizardDone(int)));
+    if (!_wizard) {
+        _wizard = new OwncloudSetupWizard(settingsDialog());
+        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, _wizard, &OwncloudSetupWizard::deleteLater);
+        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, ocApp(), &Application::slotownCloudWizardDone);
+        FolderMan::instance()->setSyncEnabled(false);
+        _wizard->startWizard();
+    }
+    raiseDialog(settingsDialog());
 }
 
 void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
@@ -1034,11 +938,11 @@ void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
         accounts.append(account.data());
     } else {
-        foreach (auto a, AccountManager::instance()->accounts()) {
+        for (const auto &a : AccountManager::instance()->accounts()) {
             accounts.append(a.data());
         }
     }
-    foreach (Folder *f, FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->map()) {
         if (accounts.contains(f->accountState())) {
             f->setSyncPaused(pause);
             if (pause) {
@@ -1050,34 +954,25 @@ void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 
 void ownCloudGui::slotShowGuiMessage(const QString &title, const QString &message)
 {
-    QMessageBox *msgBox = new QMessageBox;
+    QMessageBox *msgBox = new QMessageBox(settingsDialog());
     msgBox->setWindowFlags(msgBox->windowFlags() | Qt::WindowStaysOnTopHint);
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
     msgBox->setText(message);
     msgBox->setWindowTitle(title);
     msgBox->setIcon(QMessageBox::Information);
     msgBox->open();
+    raiseDialog(msgBox);
 }
 
 void ownCloudGui::slotShowSettings()
 {
-    if (_settingsDialog.isNull()) {
-        _settingsDialog = new SettingsDialog(this);
-        _settingsDialog->setAttribute(Qt::WA_DeleteOnClose, true);
-        _settingsDialog->show();
-    }
-    raiseDialog(_settingsDialog.data());
-}
-
-void ownCloudGui::slotSettingsDialogActivated()
-{
-    emit isShowingSettingsDialog();
+    raiseDialog(_settingsDialog);
 }
 
 void ownCloudGui::slotShowSyncProtocol()
 {
     slotShowSettings();
-    //_settingsDialog->showActivityPage();
+    _settingsDialog->showActivityPage();
 }
 
 
@@ -1087,25 +982,15 @@ void ownCloudGui::slotShutdown()
     // that saving the geometries happens ASAP during a OS shutdown
 
     // those do delete on close
-    if (!_settingsDialog.isNull())
-        _settingsDialog->close();
-    if (!_logBrowser.isNull())
-        _logBrowser->deleteLater();
+    _settingsDialog->close();
 }
 
 void ownCloudGui::slotToggleLogBrowser()
 {
-    if (_logBrowser.isNull()) {
-        // init the log browser.
-        _logBrowser = new LogBrowser;
-        // ## TODO: allow new log name maybe?
-    }
-
-    if (_logBrowser->isVisible()) {
-        _logBrowser->hide();
-    } else {
-        raiseDialog(_logBrowser);
-    }
+    auto logBrowser = new LogBrowser(settingsDialog());
+    logBrowser->setAttribute(Qt::WA_DeleteOnClose);
+    logBrowser->open();
+    raiseDialog(logBrowser);
 }
 
 void ownCloudGui::slotOpenOwnCloud()
@@ -1122,52 +1007,49 @@ void ownCloudGui::slotHelp()
 
 void ownCloudGui::raiseDialog(QWidget *raiseWidget)
 {
-    if (raiseWidget && !raiseWidget->parentWidget()) {
-        // Qt has a bug which causes parent-less dialogs to pop-under.
-        raiseWidget->showNormal();
-        raiseWidget->raise();
-        raiseWidget->activateWindow();
-
-#if defined(Q_OS_X11)
-        WId wid = widget->winId();
-        NETWM::init();
-
-        XEvent e;
-        e.xclient.type = ClientMessage;
-        e.xclient.message_type = NETWM::NET_ACTIVE_WINDOW;
-        e.xclient.display = QX11Info::display();
-        e.xclient.window = wid;
-        e.xclient.format = 32;
-        e.xclient.data.l[0] = 2;
-        e.xclient.data.l[1] = QX11Info::appTime();
-        e.xclient.data.l[2] = 0;
-        e.xclient.data.l[3] = 0l;
-        e.xclient.data.l[4] = 0l;
-        Display *display = QX11Info::display();
-        XSendEvent(display,
-            RootWindow(display, DefaultScreen(display)),
-            False, // propagate
-            SubstructureRedirectMask | SubstructureNotifyMask,
-            &e);
-#endif
+    auto window = ocApp()->gui()->settingsDialog();
+    OC_ASSERT(window);
+    OC_ASSERT_X(!qobject_cast<QDialog *>(raiseWidget) || raiseWidget->parentWidget() == window, "raiseDialog should only be called with modal dialogs");
+    if (!window) {
+        return;
     }
+    window->showNormal();
+    window->raise();
+    raiseWidget->showNormal();
+    raiseWidget->raise();
+    window->activateWindow();
+    raiseWidget->activateWindow();
+
+#if defined(Q_OS_WIN)
+    // Windows disallows raising a Window when you're not the active application.
+    // Use a common hack to attach to the active application
+    const auto activeProcessId = GetWindowThreadProcessId(GetForegroundWindow(), nullptr);
+    if (activeProcessId != qApp->applicationPid()) {
+        const auto threadId = GetCurrentThreadId();
+        // don't step here with a debugger...
+        if (AttachThreadInput(threadId, activeProcessId, true))
+        {
+            const auto hwnd = reinterpret_cast<HWND>(window->winId());
+            SetForegroundWindow(hwnd);
+            SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
+            AttachThreadInput(threadId, activeProcessId, false);
+        }
+    }
+#endif
 }
 
 
 void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &localPath, ShareDialogStartPage startPage)
 {
-    const auto folder = FolderMan::instance()->folderForPath(localPath);
+    QString file;
+    const auto folder = FolderMan::instance()->folderForPath(localPath, &file);
     if (!folder) {
         qCWarning(lcApplication) << "Could not open share dialog for" << localPath << "no responsible folder found";
         return;
     }
 
-    // For https://github.com/owncloud/client/issues/3783
-    _settingsDialog->hide();
-
     const auto accountState = folder->accountState();
 
-    const QString file = localPath.mid(folder->cleanPath().length() + 1);
     SyncJournalFileRecord fileRecord;
 
     bool resharingAllowed = true; // lets assume the good
@@ -1188,7 +1070,7 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
         | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
         | SharePermissionShare;
     if (!resharingAllowed) {
-        maxSharingPermissions = nullptr;
+        maxSharingPermissions = SharePermission(0);
     }
 
 
@@ -1198,12 +1080,13 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
         w = _shareDialogs[localPath];
     } else {
         qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
-        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, fileRecord.numericFileId(), startPage);
+        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
         w->setAttribute(Qt::WA_DeleteOnClose, true);
 
         _shareDialogs[localPath] = w;
         connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
     }
+    w->open();
     raiseDialog(w);
 }
 
@@ -1218,5 +1101,15 @@ void ownCloudGui::slotRemoveDestroyedShareDialogs()
     }
 }
 
+void ownCloudGui::slotAbout()
+{
+    if(!_aboutDialog) {
+        _aboutDialog = new AboutDialog(_settingsDialog);
+        _aboutDialog->setAttribute(Qt::WA_DeleteOnClose);
+        _aboutDialog->open();
+    }
+    raiseDialog(_aboutDialog);
+}
+
 
 } // end namespace
